unit ufmWait;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, zSvcResource, zSvcTypes, SyncObjs;

type
  TWaitThread = class(TThread)
  private
    { Private declarations }
    FSvcName: String;
    FInterval: Integer;
    FStatus: TSvcStatus;
    FLabelSet: TSvcLabelSet;
  protected
    procedure Execute; override;
    procedure UpdateWaitForm;
  public
    property SvcName: string read FSvcName write FSvcName;
    property Interval: Integer read FInterval write FInterval;
  end;

  TfmWait = class(TForm)
    tmr1: TTimer;
    pb1: TProgressBar;
    lbl1: TLabel;
    procedure tmr1Timer(Sender: TObject);
  private
    { Private declarations }
    function GetPhrase(ALabelSet: TSvcLabelSet; AStatus: TSvcStatus): string;
  public
    { Public declarations }

  end;

var
  crsWait: TCriticalSection;

procedure ShowWait(AServiceName: string; AInterval: Integer; AStatus: TSvcStatus; ALabelSet: TSvcLabelSet);
procedure ShowWaitEx(AServiceName: string; AInterval: Integer; AStatus: TSvcStatus; ALabelSet: TSvcLabelSet = slsRussian);

implementation

{$R *.dfm}
procedure ShowWait(AServiceName: string; AInterval: Integer; AStatus: TSvcStatus; ALabelSet: TSvcLabelSet);
begin
  with TWaitThread.Create(True) do
  try
    SvcName := AServiceName;
    Interval := AInterval;
    FLabelSet := ALabelSet;
    FStatus := AStatus;
    FreeOnTerminate := True;
    Resume;
  except

  end;
end;

procedure ShowWaitEx(AServiceName: string; AInterval: Integer; AStatus: TSvcStatus; ALabelSet: TSvcLabelSet = slsRussian);
begin
  with TfmWait.Create(nil) do
  try
    lbl1.Caption := GetPhrase(ALabelSet, AStatus) + sLineBreak + sLineBreak + AServiceName;
    pb1.Max := AInterval;
    ShowModal;
  finally
    Free;
  end;
end;

function TfmWait.GetPhrase(ALabelSet: TSvcLabelSet;
  AStatus: TSvcStatus): string;
begin
  case ALabelSet of
    slsRussian: Result := TSvcStatusNameRu[AStatus];
    slsEnglish: Result := TSvcStatusNameEn[AStatus];
  else
    Result := '';
  end;
end;

procedure TfmWait.tmr1Timer(Sender: TObject);
begin
  pb1.Position := pb1.Position + 100;
  if pb1.Max <= pb1.Position then
    Close;
end;

{ TWaitThread }


procedure TWaitThread.Execute;
begin
  inherited;
  crsWait.Enter;
  Synchronize(UpdateWaitForm);
  crsWait.Leave;
end;

procedure TWaitThread.UpdateWaitForm;
begin
  ShowWaitEx(SvcName, Interval, FStatus, FLabelSet);
end;

initialization
  crsWait := TCriticalSection.Create;

finalization
  if Assigned(crsWait) then
    FreeAndNil(crsWait);
end.
