unit zSvcComponent;

interface


uses
  Classes,
  ExtCtrls,
  StdCtrls,
  ActnList,
  SyncObjs,
  zSvcTypes;

type
  TSvcControler = class;

  TSvcActionSet = class(TComponent)
  private
    FStartAction:   TCustomAction;
    FStopAction:    TCustomAction;
    FPauseAction:   TCustomAction;
    FRestartAction: TCustomAction;
    FControler: TSvcControler;
    procedure UpdateButtons;
    procedure StartOnExecute(Sender: TObject);
    procedure StopOnExecute(Sender: TObject);
    procedure PauseOnExecute(Sender: TObject);
    procedure RestartOnExecute(Sender: TObject);
    procedure SetEvents;
    procedure SetControler(AValue: TSvcControler);
    procedure SetAction(AIndex: Integer; AValue: TCustomAction);
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation);override;
  public
    property Controler: TSvcControler read FControler write SetControler;
    constructor Create(AOwner: TComponent);override;
  published
    property Start:   TCustomAction index 0 read FStartAction   write SetAction;
    property Restart: TCustomAction index 1 read FRestartAction write SetAction;
    property Stop:    TCustomAction index 2 read FStopAction    write SetAction;
    property Pause:   TCustomAction index 3 read FPauseAction   write SetAction;
  end;

  TSvcThreadControler = class(TThread)
  private
    { Private declarations }
    FCommand: TSvcCommand;
    FSvcName: String;
    FControler: TSvcControler;
  protected
    procedure Execute; override;
    procedure UpdateCommand;
  public
    property SvcName: string read FSvcName write FSvcName;
    constructor Create(CreateSuspended: Boolean; ACommand: TSvcCommand; AControler: TSvcControler);
  end;

  TSvcControler = class(TComponent)
  private
    FSvcName: string;
    FTimer: TTimer;
    FOnBeforeStatusUpdate: TNotifyEvent;
    FOnAfterStatusUpdate: TStatusAfterEvent;
    FOnBeforeSvcStart: TNotifyEvent;
    FOnAfterSvcStart: TNotifyEvent;
    FStatus: TSvcStatus;
    FLabel: TLabel;
    FActionSet: TSvcActionSet;
    FAccessControl: TSvcAccesses;
    FRunType: TSvcRunType;
    FLabelSet: TSvcLabelSet;
    FComboRunType: TComboBox;
    FRunTypeInLabel: Boolean;
    FExePath: string;
    function GetStatus: TSvcStatus;
    procedure UpdateStatus;
    procedure UpdateButtonSet;
    procedure UpdateLabel;
    procedure UpdateCombo;
    procedure CreateTimer;
    procedure TimerOnTimer(Sender: TObject);
    procedure SetSvcName(AValue: string);
    procedure SetActionSet(AValue: TSvcActionSet);
    procedure SetLabel(AValue: TLabel);
    procedure SetComboRunType(AValue: TComboBox);
    procedure SetRunType(AValue: TSvcRunType);
    procedure SetRunTypeInLabel(AValue: Boolean);
    procedure SetLabelSet(AValue: TSvcLabelSet);
    procedure ComboRunTypeOnChange(Sender: TObject);
    procedure CreateCommandThread(ACommand: TSvcCommand);
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
    procedure Start;
    procedure Stop;
    procedure Pause;
    procedure Restart;
    procedure Update;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation);override;
  published
    property ServiceName:    string  read FSvcName        write SetSvcName;
    property ExePath:        string  read FExePath;
    property RunTypeInLabel: Boolean read FRunTypeInLabel write SetRunTypeInLabel default False;
    property OnBeforeStatusUpdate: TNotifyEvent      read FOnBeforeStatusUpdate write FOnBeforeStatusUpdate stored true;
    property OnAfterStatusUpdate:  TStatusAfterEvent read FOnAfterStatusUpdate  write FOnAfterStatusUpdate  stored true;
    property OnBeforeSvcStart:     TNotifyEvent      read FOnBeforeSvcStart     write FOnBeforeSvcStart     stored true;
    property OnAfterSvcStart:      TNotifyEvent      read FOnAfterSvcStart      write FOnAfterSvcStart      stored true;
    property Status:      TSvcStatus   read GetStatus;
    property AccessTypes: TSvcAccesses read FAccessControl;
    property RunType:     TSvcRunType  read FRunType;
    property ServiceLabel: TLabel        read FLabel        write SetLabel;
    property Actions:      TSvcActionSet read FActionSet    write SetActionSet;
    property LabelSet:     TSvcLabelSet  read FLabelSet     write SetLabelSet stored False nodefault;
    property ComboRunType: TComboBox     read FComboRunType write SetComboRunType;
  end;

var
  crsCommand: TCriticalSection;

implementation

uses SysUtils,
     Windows,
     Dialogs,
     zSvcResource,
     zSvcManager;

{ TSvcControler }

procedure TSvcControler.ComboRunTypeOnChange(Sender: TObject);
begin
  if Assigned(FComboRunType) then
  begin
    SetRunType(TSvcRunType(FComboRunType.ItemIndex + 2));
  end;
end;

constructor TSvcControler.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  CreateTimer;
end;

procedure TSvcControler.CreateCommandThread(ACommand: TSvcCommand);
begin
  with TSvcThreadControler.Create(True, ACommand, Self) do
  try
    FreeOnTerminate := True;
    SvcName := ServiceName;
    Resume;
  except
    on E: Exception do
      MessageDlg(E.Message,mtError, [mbOk],0);
  end;
end;

procedure TSvcControler.CreateTimer;
begin
  FTimer := TTimer.Create(Self);
  with FTimer do
  begin
    OnTimer := TimerOnTimer;
    Interval := 100;
    Enabled := True;
  end;
end;

destructor TSvcControler.Destroy;
begin
  FreeAndNil(FTimer);
  inherited;
end;


function TSvcControler.GetStatus: TSvcStatus;
begin
  Result := FStatus;
end;

procedure TSvcControler.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then
  begin
    if AComponent = FLabel then
        FLabel     := nil;
    if AComponent = FActionSet then
        FActionSet := nil;
    if AComponent = FComboRunType then
      FComboRunType := nil;
  end;
  if Operation = opInsert then
    if AComponent is TSvcActionSet then
      if FActionSet = nil then
        FActionSet := TSvcActionSet(AComponent);
end;

procedure TSvcControler.Pause;
begin
  CreateCommandThread(svcPause);
end;

procedure TSvcControler.Restart;
begin
  CreateCommandThread(svcRestart);
end;

procedure TSvcControler.SetActionSet(AValue: TSvcActionSet);
begin
  if Assigned(AValue) then
  begin
    FActionSet.Controler := (Self);
    if AValue <> nil then
    begin
      RemoveFreeNotification(AValue);
      FreeNotification(AValue);
    end;
  end;
    
  FActionSet := AValue;
end;

procedure TSvcControler.SetComboRunType(AValue: TComboBox);
begin
  if FComboRunType <> AValue then
  begin
    FComboRunType := AValue;
    if AValue <> nil then
    begin
	    RemoveFreeNotification(AValue);
      FreeNotification(AValue);
      UpdateCombo;
    end;
  end;
end;

procedure TSvcControler.SetLabel(AValue: TLabel);
begin
  if FLabel <> AValue then
  begin
    FLabel := AValue;
    if AValue <> nil then
    begin
      RemoveFreeNotification(AValue);
      FreeNotification(AValue);
    end;
  end;
end;

procedure TSvcControler.SetLabelSet(AValue: TSvcLabelSet);
begin
  if FLabelSet <> AValue then
  begin
    FLabelSet := AValue;
    UpdateButtonSet;
    UpdateLabel;
  end;
end;

procedure TSvcControler.SetRunType(AValue: TSvcRunType);
begin
  if FRunType <> AValue then
  begin
    SetServiceRunType(EmptyStr, FSvcName, AValue);
    UpdateStatus;
  end;
end;

procedure TSvcControler.SetRunTypeInLabel(AValue: Boolean);
begin
  if FRunTypeInLabel <> AValue then
  begin
    FRunTypeInLabel := AValue;
    UpdateLabel;
  end;
end;

procedure TSvcControler.SetSvcName(AValue: string);
begin
  if FSvcName = AValue then
    Exit;
  FSvcName := AValue;
  UpdateStatus;
  UpdateLabel;
  UpdateCombo;
end;

procedure TSvcControler.Start;
begin
  if Assigned(FOnBeforeSvcStart) then
    try
	  FTimer.Enabled := False;
      FOnBeforeSvcStart(Self);
	finally
	  FTimer.Enabled := True;
	end;
  case FStatus of
    svsStopped: CreateCommandThread(svcStart);
    svsPaused:  CreateCommandThread(svcContinue);
  end;
  if Assigned(FOnAfterSvcStart) then
    try
	  FTimer.Enabled := False;
      FOnAfterSvcStart(Self);
	finally
	  FTimer.Enabled := True;
	end;
end;

procedure TSvcControler.Stop;
begin
  CreateCommandThread(svcStop);
end;

procedure TSvcControler.TimerOnTimer(Sender: TObject);
begin
  UpdateStatus;
end;

procedure TSvcControler.Update;
begin
  UpdateStatus;
end;

procedure TSvcControler.UpdateButtonSet;
begin
  if Assigned(FActionSet) then
    FActionSet.UpdateButtons;
end;

procedure TSvcControler.UpdateCombo;
begin
  if Assigned(FComboRunType) then
    with FComboRunType do
    begin
      Style := csDropDownList;
      OnChange := ComboRunTypeOnChange;
      Clear;
      case LabelSet of
        slsRussian:
        begin
          Items.Add(SAutoStartRu);
          Items.Add(SManualStartRu);
          Items.Add(SDisabledStartRu);
          Items.Add(SNotInstalledRu);
        end;
        slsEnglish:
        begin
          Items.Add(SAutoStartEn);
          Items.Add(SManualStartEn);
          Items.Add(SDisabledStartEn);
          Items.Add(SNotInstalledEn);
        end;
      end;
      Enabled := not (FStatus in [svsNotInstalled]);
      if FStatus in [svsNotInstalled] then
      begin
        ItemIndex := 3;
      end
      else
      begin
        Items.Delete(3);
        ItemIndex := Ord(FRunType) - 2;
      end;
    end;
end;

procedure TSvcControler.UpdateLabel;
begin
  if Assigned(FLabel) then
  if not FRunTypeInLabel then
    case FLabelSet of
      slsRussian: FLabel.Caption := TSvcStatusNameRu[FStatus];
      slsEnglish: FLabel.Caption := TSvcStatusNameEn[FStatus];
    end
  else
    case FLabelSet of
      slsRussian: FLabel.Caption := TSvcStatusNameRu[FStatus] + ' - ' + TSvcRunTypeNameRu[FRunType];
      slsEnglish: FLabel.Caption := TSvcStatusNameEn[FStatus] + ' - ' + TSvcRunTypeNameEn[FRunType];
    end;
end;

procedure TSvcControler.UpdateStatus;
var
  vSvcInfo: TSvcInfo;
begin
  if Assigned(FOnBeforeStatusUpdate) then
   try
     FTimer.Enabled := False;
     FOnBeforeStatusUpdate(Self);
   finally
     FTimer.Enabled := True;
   end;

  if FSvcName = EmptyStr then
    Exit;
  GetServiceInfo(EmptyStr, FSvcName, vSvcInfo);

  with vSvcInfo do
  begin
    FAccessControl := Accesses;
    FExePath       := Path;
    if (FStatus <> Status) or (FRunType <> Runtype) then
    begin
      FStatus := Status;
      FRunType := RunType;
      UpdateCombo;
    end;
    UpdateLabel;
  end;

  UpdateButtonSet;

  if Assigned(FOnAfterStatusUpdate) then
    try
      FTimer.Enabled := False;
      FOnAfterStatusUpdate(Self, FStatus);
    finally
      FTimer.Enabled := True;
    end;
end;

{ TSvcButtonSet }

constructor TSvcActionSet.Create(AOwner: TComponent);
begin
  inherited;
    if Assigned(Pause) then
    Pause.Enabled := false;
  if Assigned(Start) then
    Start.Enabled := false;
  if Assigned(Restart) then
    Restart.Enabled := false;
  if Assigned(Stop) then
    Stop.Enabled := false;
end;

procedure TSvcActionSet.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then
  begin
    if AComponent = FStartAction then
      FStartAction := nil;
    if AComponent = FStopAction then
      FStopAction := nil;
    if AComponent = FPauseAction then
      FPauseAction := nil;
    if AComponent = FRestartAction then
      FRestartAction := nil;
  end;
end;

procedure TSvcActionSet.PauseOnExecute(Sender: TObject);
begin
  if Assigned(FControler) then
    FControler.Pause;
end;

procedure TSvcActionSet.RestartOnExecute(Sender: TObject);
begin
  if Assigned(FControler) then
    FControler.Restart;
end;

procedure TSvcActionSet.SetAction(AIndex: Integer; AValue: TCustomAction);
var
  tmpAct: TCustomAction;
begin
  case AIndex of
    0: tmpAct := FStartAction;
    1: tmpAct := FRestartAction;
    2: tmpAct := FStopAction;
    3: tmpAct := FPauseAction;
  else
    tmpAct := nil;
  end;
  if tmpAct <> AValue then
  begin
    if AValue <> nil then
    begin
      RemoveFreeNotification(AValue);
      FreeNotification(AValue);
    end;
    case AIndex of
      0: FStartAction   := AValue;
      1: FRestartAction := AValue;
      2: FStopAction    := AValue;
      3: FPauseAction   := AValue;
    else
      Exit;
    end;
  end;
end;

procedure TSvcActionSet.SetControler(AValue: TSvcControler);
begin
  if Assigned(AValue) then
  begin
    FControler := AValue;
  end;
end;

procedure TSvcActionSet.SetEvents;
begin
  if Assigned(FPauseAction)   then
    FPauseAction.OnExecute   := PauseOnExecute;
  if Assigned(FStartAction)   then
    FStartAction.OnExecute   := StartOnExecute;
  if Assigned(FRestartAction) then
    FRestartAction.OnExecute := RestartOnExecute;
  if Assigned(FStopAction)    then
    FStopAction.OnExecute    := StopOnExecute;
end;

procedure TSvcActionSet.StartOnExecute(Sender: TObject);
begin
  if Assigned(FControler) then
    FControler.Start;
end;

procedure TSvcActionSet.StopOnExecute(Sender: TObject);
begin
  if Assigned(FControler) then
    FControler.Stop;
end;

procedure TSvcActionSet.UpdateButtons;
begin
  if not Assigned(Controler) then
    Exit;
  with Controler do
  begin
    if Assigned(FStartAction) then
      FStartAction.Enabled := (Status in [svsStopped, svsPaused]) and
                            not(RunType in [srtDisabled]);
    if Assigned(FStopAction) then
      FStopAction.Enabled := (Status in [svsStarted, svsPaused]) and
                           (scaStop in AccessTypes) and
                           ((not(RunType in [srtDisabled]) or (Status in [svsStarted, svsPaused])));
    if Assigned(FRestartAction) then
      FRestartAction.Enabled := (Status in [svsStarted, svsPaused]) and
                              (scaStop in AccessTypes) and
                              not(RunType in [srtDisabled]);
    if Assigned(FPauseAction) then
      FPauseAction.Enabled := (Status in [svsStarted]) and
                            (scaPauseContinue in AccessTypes) and
                            not(RunType in [srtDisabled]);
  end;
  SetEvents;
end;

{ TSvcThreadControler }

constructor TSvcThreadControler.Create(CreateSuspended: Boolean; ACommand: TSvcCommand; AControler: TSvcControler);
begin
  inherited Create(CreateSuspended);
  FCommand := ACommand;
  FControler := AControler;
end;

procedure TSvcThreadControler.Execute;
begin
  inherited;
  UpdateCommand;
end;

procedure TSvcThreadControler.UpdateCommand;
begin
  crsCommand.Enter;
  case FCommand of
    svcStart:    ServiceStart(   EmptyStr, SvcName, FControler.LabelSet);
    svcStop:     ServiceStop(    EmptyStr, SvcName, FControler.LabelSet);
    svcPause:    ServicePause(   EmptyStr, SvcName, FControler.LabelSet);
    svcContinue: ServiceContinue(EmptyStr, SvcName, FControler.LabelSet);
    svcRestart:
    begin
      ServiceStop( EmptyStr, SvcName, FControler.LabelSet);
      Sleep(500);
      ServiceStart(EmptyStr, SvcName, FControler.LabelSet);
    end;
  end;
  crsCommand.Leave;
end;


initialization
  crsCommand := TCriticalSection.Create;


finalization
  if Assigned(crsCommand) then
    FreeAndNil(crsCommand);
end.
