unit zSvcManager;

interface

uses Windows, zSvcTypes;

function ServiceStart(AMachine, AServiceName: string; ALabelSet: TSvcLabelSet): boolean;
function ServiceContinue(AMachine,AServiceName: string; ALabelSet: TSvcLabelSet): boolean;
function ServicePause(AMachine,AServiceName: string; ALabelSet: TSvcLabelSet): boolean;
function ServiceStop(AMachine,AServiceName: string; ALabelSet: TSvcLabelSet): boolean;

procedure GetServiceInfo(sMachine, sService: string; var SvcInfo: TSvcInfo);
function SetServiceRunType(AMachine, AServiceName: string; ARunType: TSvcRunType): boolean;

implementation

uses
  WinSvc, ufmWait;

function SetServiceRunType(AMachine, AServiceName: string; ARunType: TSvcRunType): boolean;
// aMachine  UNC ,     
var
  h_manager,h_svc: SC_Handle;
begin
  Result := False;
  h_manager := OpenSCManager(PChar(AMachine), nil, SC_MANAGER_ALL_ACCESS);
  if h_manager > 0 then
  begin
    h_svc := OpenService(h_manager, PChar(AServiceName), SERVICE_QUERY_STATUS or SERVICE_QUERY_CONFIG or SERVICE_CHANGE_CONFIG);
    if h_svc > 0 then
    begin
      Result := ChangeServiceConfig(h_svc, SERVICE_NO_CHANGE, Ord(ARunType), SERVICE_NO_CHANGE, nil, nil, nil, nil, nil, nil, nil);
      CloseServiceHandle(h_svc);
    end;
    CloseServiceHandle(h_manager);
  end;
end;

procedure GetServiceInfo(sMachine, sService: string; var SvcInfo: TSvcInfo);
var
  h_manager, h_svc: SC_HANDLE;
  svc_status: TServiceStatus;
  svc_config: PQueryServiceConfig;
  vSize: DWORD;
  Ret: BOOL;
begin
  SvcInfo.Accesses := [scaShutdown];
  SvcInfo.RunType  := srtDisabled;
  SvcInfo.Status   := svsNotInstalled;
  h_manager := OpenSCManager(PChar(sMachine) ,nil, SC_MANAGER_CONNECT);
  if h_manager > 0 then
  begin
    h_svc := OpenService(h_manager,PChar(sService), SERVICE_QUERY_CONFIG or SERVICE_QUERY_STATUS);
    if h_svc > 0 then
    begin

      QueryServiceConfig(h_svc, nil, 0, vSize);
      svc_config := nil;
      vSize := 4096;
      repeat
        ReallocMem(svc_config, vSize);
        Ret := QueryServiceConfig(h_svc, svc_config, vSize, vSize);
      until Ret or (GetLastError <> ERROR_INSUFFICIENT_BUFFER);
      if (QueryServiceConfig(h_svc, svc_config, vSize, vSize)) then
      begin
        SvcInfo.RunType := TSvcRunType(svc_config^.dwStartType);
        SvcInfo.Path := String(svc_config^.lpBinaryPathName);
        FreeMem(svc_config);
      end;

      if QueryServiceStatus(h_svc, svc_status) then
      begin
        SvcInfo.Status := TSvcStatus(svc_status.dwCurrentState);
        if (SERVICE_ACCEPT_STOP and svc_status.dwControlsAccepted) > 0 then
          Include(SvcInfo.Accesses, scaStop);
        if (SERVICE_ACCEPT_PAUSE_CONTINUE and svc_status.dwControlsAccepted) > 0 then
          Include(SvcInfo.Accesses, scaPauseContinue);
      end;
      CloseServiceHandle(h_svc);
    end;
    CloseServiceHandle(h_manager);
  end;
end;

//******************************************************************************
function ServiceStart(AMachine, AServiceName: string;
  ALabelSet: TSvcLabelSet): boolean;
// aMachine  UNC ,     
var
  h_manager, h_svc: SC_HANDLE;
  svc_status: TServiceStatus;
  Temp: PChar;
  dwCheckPoint: DWORD;
begin
  svc_status.dwCurrentState := 1;
  h_manager := OpenSCManager(PChar(AMachine), nil, SC_MANAGER_CONNECT);
  if h_manager > 0 then
  begin
    h_svc := OpenService(h_manager, PChar(AServiceName),
    SERVICE_START or SERVICE_QUERY_STATUS);
    if h_svc > 0 then
    begin
      temp := nil;
      if (StartService(h_svc, 0, temp)) then
        if (QueryServiceStatus(h_svc,svc_status)) then
        begin
          while (SERVICE_RUNNING <> svc_status.dwCurrentState) do
          begin
            dwCheckPoint := svc_status.dwCheckPoint;
            ShowWait(AServiceName,
                     svc_status.dwWaitHint,
                     svsStartPending,
                     ALabelSet);
            Sleep(svc_status.dwWaitHint);
            if (not QueryServiceStatus(h_svc,svc_status)) then
              Break;
            if (svc_status.dwCheckPoint < dwCheckPoint) then
            begin
              // QueryServiceStatus   dwCheckPoint
              Break;
            end;
          end;
        end;
      CloseServiceHandle(h_svc);
    end;
    CloseServiceHandle(h_manager);
  end;
  Result := SERVICE_RUNNING = svc_status.dwCurrentState;
end;

function ServiceStop(AMachine,AServiceName: string;
  ALabelSet: TSvcLabelSet): Boolean;
// aMachine  UNC ,     
var
  h_manager, h_svc: SC_HANDLE;
  svc_status: TServiceStatus;
  dwCheckPoint: DWORD;
begin
  h_manager:=OpenSCManager(PChar(AMachine),nil, SC_MANAGER_CONNECT);
  if h_manager > 0 then
  begin
    h_svc := OpenService(h_manager,PChar(AServiceName),
    SERVICE_STOP or SERVICE_QUERY_STATUS);
    if h_svc > 0 then
    begin
      if(ControlService(h_svc,SERVICE_CONTROL_STOP, svc_status))then
      begin
        if(QueryServiceStatus(h_svc,svc_status))then
        begin
          while(SERVICE_STOPPED <> svc_status.dwCurrentState)do
          begin
            dwCheckPoint := svc_status.dwCheckPoint;
            ShowWait(AServiceName, svc_status.dwWaitHint, svsStopPending, ALabelSet);
            Sleep(svc_status.dwWaitHint);
            if(not QueryServiceStatus(h_svc,svc_status))then
            begin
              // couldn't check status
              Break;
            end;
            if(svc_status.dwCheckPoint < dwCheckPoint)then
              Break;
          end;
        end;
      end;
      CloseServiceHandle(h_svc);
    end;
    CloseServiceHandle(h_manager);
  end;
  Result := SERVICE_STOPPED = svc_status.dwCurrentState;
end;

function ServicePause(AMachine,AServiceName: string;
  ALabelSet: TSvcLabelSet): Boolean;
// aMachine  UNC ,     
var
  h_manager, h_svc: SC_HANDLE;
  svc_status: TServiceStatus;
  dwCheckPoint: DWORD;
begin
  h_manager:=OpenSCManager(PChar(AMachine),nil, SC_MANAGER_CONNECT);
  if h_manager > 0 then
  begin
    h_svc := OpenService(h_manager,PChar(AServiceName),
    SERVICE_PAUSE_CONTINUE or SERVICE_QUERY_STATUS);
    if h_svc > 0 then
    begin
      if(ControlService(h_svc,SERVICE_CONTROL_PAUSE, svc_status))then
      begin
        if(QueryServiceStatus(h_svc,svc_status))then
        begin
          while(SERVICE_PAUSED <> svc_status.dwCurrentState)do
          begin
            dwCheckPoint := svc_status.dwCheckPoint;
            ShowWait(AServiceName, svc_status.dwWaitHint, svsPausePending, ALabelSet);
            Sleep(svc_status.dwWaitHint);
            if(not QueryServiceStatus(h_svc,svc_status))then
            begin
              // couldn't check status
              Break;
            end;
            if(svc_status.dwCheckPoint < dwCheckPoint)then
              Break;
          end;
        end;
      end;
      CloseServiceHandle(h_svc);
    end;
    CloseServiceHandle(h_manager);
  end;
  Result := SERVICE_PAUSED = svc_status.dwCurrentState;
end;

function ServiceContinue(AMachine,AServiceName: string;
  ALabelSet: TSvcLabelSet): Boolean;
// aMachine  UNC ,     
var
  h_manager, h_svc: SC_HANDLE;
  svc_status: TServiceStatus;
  dwCheckPoint: DWORD;
begin
  h_manager:=OpenSCManager(PChar(AMachine),nil, SC_MANAGER_CONNECT);
  if h_manager > 0 then
  begin
    h_svc := OpenService(h_manager,PChar(AServiceName),
    SERVICE_PAUSE_CONTINUE or SERVICE_QUERY_STATUS);
    if h_svc > 0 then
    begin
      if(ControlService(h_svc,SERVICE_CONTROL_CONTINUE, svc_status))then
      begin
        if(QueryServiceStatus(h_svc,svc_status))then
        begin
          while(SERVICE_RUNNING <> svc_status.dwCurrentState)do
          begin
            dwCheckPoint := svc_status.dwCheckPoint;
            ShowWait(AServiceName, svc_status.dwWaitHint, svsContinuePending, ALabelSet);
            Sleep(svc_status.dwWaitHint);
            if(not QueryServiceStatus(h_svc,svc_status))then
            begin
              // couldn't check status
              Break;
            end;
            if(svc_status.dwCheckPoint < dwCheckPoint)then
              Break;
          end;
        end;
      end;
      CloseServiceHandle(h_svc);
    end;
    CloseServiceHandle(h_manager);
  end;
  Result := SERVICE_RUNNING = svc_status.dwCurrentState;
end;

end.
